/*
 * Created on Nov 11, 2004
 *
 * @author Fabio Zadrozny
 */
package org.python.pydev.editor.codecompletion.revisited.visitors;

import java.util.HashSet;
import java.util.Iterator;

import org.python.pydev.core.IToken;
import org.python.pydev.editor.codecompletion.revisited.modules.SourceToken;
import org.python.pydev.parser.jython.SimpleNode;
import org.python.pydev.parser.jython.ast.Assign;
import org.python.pydev.parser.jython.ast.ClassDef;
import org.python.pydev.parser.jython.ast.FunctionDef;
import org.python.pydev.parser.jython.ast.Import;
import org.python.pydev.parser.jython.ast.ImportFrom;
import org.python.pydev.parser.jython.ast.List;
import org.python.pydev.parser.jython.ast.Name;
import org.python.pydev.parser.jython.ast.Str;
import org.python.pydev.parser.jython.ast.exprType;

/**
 * This class visits only the global context. Other visitors should visit contexts inside of this one.
 * 
 * @author Fabio Zadrozny
 */
public class GlobalModelVisitor extends AbstractVisitor {

    private int visitWhat;
    private SourceToken __all__;
    private Assign __all__Assign;
    private Assign lastAssign;
	private boolean onlyAllowTokensIn__all__;

    /**
     * @param moduleName
     * @param global_tokens2
     */
    public GlobalModelVisitor(int visitWhat, String moduleName, boolean onlyAllowTokensIn__all__) {
        this.visitWhat = visitWhat;
        this.moduleName = moduleName;
        this.onlyAllowTokensIn__all__ = onlyAllowTokensIn__all__;
        if(moduleName != null && moduleName.endsWith("__init__")){
            this.tokens.add(new SourceToken(new Name("__path__", Name.Load), "__path__", "", "", moduleName));
        }
    }

    protected Object unhandled_node(SimpleNode node) throws Exception {
        return null;
    }

    public void traverse(SimpleNode node) throws Exception {
        node.traverse(this);
    }

    public Object visitClassDef(ClassDef node) throws Exception {
        //when visiting the global namespace, we don't go into any inner scope.
        if ((this.visitWhat & GLOBAL_TOKENS) != 0) {
            addToken(node);
        } 
        return null;
    }

    public Object visitFunctionDef(FunctionDef node) throws Exception {
        //when visiting the global namespace, we don't go into any inner scope.
        if ((this.visitWhat & GLOBAL_TOKENS) != 0) {
            addToken(node);
        }
        return null;
    }

    /**
     * Name should be whithin assign.
     * 
     * @see org.python.pydev.parser.jython.ast.VisitorIF#visitAssign(org.python.pydev.parser.jython.ast.Assign)
     */
    public Object visitAssign(Assign node) throws Exception {
    	lastAssign = node;
        node.traverse(this);
        return null;
    }

    /**
     * Visiting some name
     * 
     * @see org.python.pydev.parser.jython.ast.VisitorIF#visitName(org.python.pydev.parser.jython.ast.Name)
     */
    public Object visitName(Name node) throws Exception {
        //when visiting the global namespace, we don't go into any inner scope.
        if ((this.visitWhat & GLOBAL_TOKENS) != 0) {
            if (node.ctx == Name.Store) {
                SourceToken added = addToken(node);
                if(onlyAllowTokensIn__all__ && added.getRepresentation().equals("__all__")){
                	__all__ = added;
                	__all__Assign = lastAssign;
                }
            }
        }
        return null;
    }

    /**
     * Visiting some import from
     * @see org.python.pydev.parser.jython.ast.VisitorBase#visitImportFrom(org.python.pydev.parser.jython.ast.ImportFrom)
     */
    public Object visitImportFrom(ImportFrom node) throws Exception {
        if ((this.visitWhat & WILD_MODULES) != 0) {
            makeWildImportToken(node, this.tokens, moduleName);
        } 
        
        if ((this.visitWhat & ALIAS_MODULES) != 0) {
            makeImportToken(node, this.tokens, moduleName, true);
        }
        return null;
    }



    /**
     * Visiting some import
     * @see org.python.pydev.parser.jython.ast.VisitorBase#visitImport(org.python.pydev.parser.jython.ast.Import)
     */
    public Object visitImport(Import node) throws Exception {
        if ((this.visitWhat & ALIAS_MODULES) != 0) {
            makeImportToken(node, this.tokens, moduleName, true);
        }
        return null;
    }

    
    /**
     * @see org.python.pydev.parser.jython.ast.VisitorBase#visitStr(org.python.pydev.parser.jython.ast.Str)
     */
    public Object visitStr(Str node) throws Exception {
        if((this.visitWhat & MODULE_DOCSTRING) != 0){
            this.tokens.add(new SourceToken(node, node.s, "", "", moduleName));
        }
        return null;
    }
    
    
    /**
     * Overridden to check __all__
     */
    @Override
    protected void finishVisit() {
    	if(__all__ != null){
    		SimpleNode ast = __all__.getAst();
    		//just checking it
    		if(__all__Assign.targets != null && __all__Assign.targets.length == 1 && __all__Assign.targets[0] == ast){
    			HashSet<String> validTokensInAll = new HashSet<String>();
    			exprType value = __all__Assign.value;
    			if(value instanceof List){
					List valueList = (List) value;
					if(valueList.elts != null){
						for(exprType elt:valueList.elts){
							if(elt instanceof Str){
								Str str = (Str) elt;
								validTokensInAll.add(str.s);
							}
						}
					}
    			}
    			
    			if(validTokensInAll.size() > 0){
    				for(Iterator<IToken> it = tokens.iterator();it.hasNext();){
    					IToken tok = it.next();
    					if(!validTokensInAll.contains(tok.getRepresentation())){
    						it.remove();
    					}
    				}
    			}
    		}
    	}
    }
}